//========================================================================
// 4x4 array of FPGA cells
//========================================================================

`ifndef BLOCK_4X4_SV
`define BLOCK_4X4_SV

`include "clb.sv"
`include "switches.sv"
`include "vc/muxes.sv"

module block_4x4
(
  (* noprune, keep = 1 *) input  logic [447:0] p,

  (* noprune, keep = 1 *) input  logic       g_clr,
  (* noprune, keep = 1 *) input  logic       g_clk,

  (* noprune, keep = 1 *) input  logic       clk_in  [3:0],
  (* noprune, keep = 1 *) input  logic       clr_in  [3:0],

  (* noprune, keep = 1 *) input  logic       n4_in   [3:0],
  (* noprune, keep = 1 *) input  logic       s4_in   [3:0],
  (* noprune, keep = 1 *) input  logic       e4_in   [3:0],
  (* noprune, keep = 1 *) input  logic       w4_in   [3:0],

  (* noprune, keep = 1 *) input  logic       pn4_in  [3:0],
  (* noprune, keep = 1 *) input  logic       ps4_in  [3:0],
  (* noprune, keep = 1 *) input  logic       pe4_in  [3:0],
  (* noprune, keep = 1 *) input  logic       pw4_in  [3:0],

  (* noprune, keep = 1 *) input  logic       n_in    [3:0],
  (* noprune, keep = 1 *) input  logic       s_in    [3:0],
  (* noprune, keep = 1 *) input  logic       e_in    [3:0],
  (* noprune, keep = 1 *) input  logic       w_in    [3:0],

  (* noprune, keep = 1 *) output logic       clk_out [3:0],
  (* noprune, keep = 1 *) output logic       clr_out [3:0],

  (* noprune, keep = 1 *) output logic       n4_out  [3:0],
  (* noprune, keep = 1 *) output logic       s4_out  [3:0],
  (* noprune, keep = 1 *) output logic       e4_out  [3:0],
  (* noprune, keep = 1 *) output logic       w4_out  [3:0],

  (* noprune, keep = 1 *) output logic       pn4_out [3:0],
  (* noprune, keep = 1 *) output logic       ps4_out [3:0],
  (* noprune, keep = 1 *) output logic       pe4_out [3:0],
  (* noprune, keep = 1 *) output logic       pw4_out [3:0],

  (* noprune, keep = 1 *) output logic       n_out   [3:0],
  (* noprune, keep = 1 *) output logic       s_out   [3:0],
  (* noprune, keep = 1 *) output logic       e_out   [3:0],
  (* noprune, keep = 1 *) output logic       w_out   [3:0]
);

  (* noprune, keep = 1 *) logic nc_out [3:0][3:0];
  (* noprune, keep = 1 *) logic sc_out [3:0][3:0];
  (* noprune, keep = 1 *) logic ec_out [3:0][3:0];
  (* noprune, keep = 1 *) logic wc_out [3:0][3:0];

  genvar i, j;
  generate
    for (i=0; i<=3; i=i+1) begin : rows
      for (j=0; j<=3; j=j+1) begin : columns
        if (i == 0) begin
          if (j == 0) begin
            clb clb
            (
              .clk_in (clk_in[j]),
              .clr_in (clr_in[j]),
              .p      (p[(i*4*23)+(j*23)+22:(i*4*23)+(j*23)]),
              .n_in   (n_in[j]),
              .s_in   (nc_out[i+1][j]),
              .e_in   (wc_out[i][j+1]),
              .w_in   (w_in[i]),
              .n4_in  (n4_in[j]),
              .s4_in  (s4_in[j]),
              .e4_in  (e4_in[i]),
              .w4_in  (w4_in[i]),
              .n_out  (nc_out[i][j]),
              .s_out  (sc_out[i][j]),
              .e_out  (ec_out[i][j]),
              .w_out  (wc_out[i][j])
            );
          end
          else if (j == 3) begin
            clb clb
            (
              .clk_in (clk_in[j]),
              .clr_in (clr_in[j]),
              .p      (p[(i*4*23)+(j*23)+22:(i*4*23)+(j*23)]),
              .n_in   (n_in[j]),
              .s_in   (nc_out[i+1][j]),
              .e_in   (e_in[i]),
              .w_in   (ec_out[i][j-1]),
              .n4_in  (n4_in[j]),
              .s4_in  (s4_in[j]),
              .e4_in  (e4_in[i]),
              .w4_in  (w4_in[i]),
              .n_out  (nc_out[i][j]),
              .s_out  (sc_out[i][j]),
              .e_out  (ec_out[i][j]),
              .w_out  (wc_out[i][j])
            );
          end
          else begin
            clb clb
            (
              .clk_in (clk_in[j]),
              .clr_in (clr_in[j]),
              .p      (p[(i*4*23)+(j*23)+22:(i*4*23)+(j*23)]),
              .n_in   (n_in[j]),
              .s_in   (nc_out[i+1][j]),
              .e_in   (wc_out[i][j+1]),
              .w_in   (ec_out[i][j-1]),
              .n4_in  (n4_in[j]),
              .s4_in  (s4_in[j]),
              .e4_in  (e4_in[i]),
              .w4_in  (w4_in[i]),
              .n_out  (nc_out[i][j]),
              .s_out  (sc_out[i][j]),
              .e_out  (ec_out[i][j]),
              .w_out  (wc_out[i][j])
            );
          end
        end

        else if (i == 3) begin
          if (j == 0) begin
            clb clb
            (
              .clk_in (clk_in[j]),
              .clr_in (clr_in[j]),
              .p      (p[(i*4*23)+(j*23)+22:(i*4*23)+(j*23)]),
              .n_in   (sc_out[i-1][j]),
              .s_in   (s_in[j]),
              .e_in   (wc_out[i][j+1]),
              .w_in   (w_in[i]),
              .n4_in  (n4_in[j]),
              .s4_in  (s4_in[j]),
              .e4_in  (e4_in[i]),
              .w4_in  (w4_in[i]),
              .n_out  (nc_out[i][j]),
              .s_out  (sc_out[i][j]),
              .e_out  (ec_out[i][j]),
              .w_out  (wc_out[i][j])
            );
          end
          else if (j == 3) begin
            clb clb
            (
              .clk_in (clk_in[j]),
              .clr_in (clr_in[j]),
              .p      (p[(i*4*23)+(j*23)+22:(i*4*23)+(j*23)]),
              .n_in   (sc_out[i-1][j]),
              .s_in   (s_in[j]),
              .e_in   (e_in[i]),
              .w_in   (ec_out[i][j-1]),
              .n4_in  (n4_in[j]),
              .s4_in  (s4_in[j]),
              .e4_in  (e4_in[i]),
              .w4_in  (w4_in[i]),
              .n_out  (nc_out[i][j]),
              .s_out  (sc_out[i][j]),
              .e_out  (ec_out[i][j]),
              .w_out  (wc_out[i][j])
            );
          end
          else begin
            clb clb
            (
              .clk_in (clk_in[j]),
              .clr_in (clr_in[j]),
              .p      (p[(i*4*23)+(j*23)+22:(i*4*23)+(j*23)]),
              .n_in   (sc_out[i-1][j]),
              .s_in   (s_in[j]),
              .e_in   (wc_out[i][j+1]),
              .w_in   (ec_out[i][j-1]),
              .n4_in  (n4_in[j]),
              .s4_in  (s4_in[j]),
              .e4_in  (e4_in[i]),
              .w4_in  (w4_in[i]),
              .n_out  (nc_out[i][j]),
              .s_out  (sc_out[i][j]),
              .e_out  (ec_out[i][j]),
              .w_out  (wc_out[i][j])
            );
          end
        end

        else begin
          if (j == 0) begin
            clb clb
            (
              .clk_in (clk_in[j]),
              .clr_in (clr_in[j]),
              .p      (p[(i*4*23)+(j*23)+22:(i*4*23)+(j*23)]),
              .n_in   (sc_out[i-1][j]),
              .s_in   (nc_out[i+1][j]),
              .e_in   (wc_out[i][j+1]),
              .w_in   (w_in[i]),
              .n4_in  (n4_in[j]),
              .s4_in  (s4_in[j]),
              .e4_in  (e4_in[i]),
              .w4_in  (w4_in[i]),
              .n_out  (nc_out[i][j]),
              .s_out  (sc_out[i][j]),
              .e_out  (ec_out[i][j]),
              .w_out  (wc_out[i][j])
            );
          end
          else if (j == 3) begin
            clb clb
            (
              .clk_in (clk_in[j]),
              .clr_in (clr_in[j]),
              .p      (p[(i*4*23)+(j*23)+22:(i*4*23)+(j*23)]),
              .n_in   (sc_out[i-1][j]),
              .s_in   (nc_out[i+1][j]),
              .e_in   (e_in[i]),
              .w_in   (ec_out[i][j-1]),
              .n4_in  (n4_in[j]),
              .s4_in  (s4_in[j]),
              .e4_in  (e4_in[i]),
              .w4_in  (w4_in[i]),
              .n_out  (nc_out[i][j]),
              .s_out  (sc_out[i][j]),
              .e_out  (ec_out[i][j]),
              .w_out  (wc_out[i][j])
            );
          end
          else begin
            clb clb
            (
              .clk_in (clk_in[j]),
              .clr_in (clr_in[j]),
              .p      (p[(i*4*23)+(j*23)+22:(i*4*23)+(j*23)]),
              .n_in   (sc_out[i-1][j]),
              .s_in   (nc_out[i+1][j]),
              .e_in   (wc_out[i][j+1]),
              .w_in   (ec_out[i][j-1]),
              .n4_in  (n4_in[j]),
              .s4_in  (s4_in[j]),
              .e4_in  (e4_in[i]),
              .w4_in  (w4_in[i]),
              .n_out  (nc_out[i][j]),
              .s_out  (sc_out[i][j]),
              .e_out  (ec_out[i][j]),
              .w_out  (wc_out[i][j])
            );
          end
        end
      end
    end

    for (j=0; j<=3; j=j+1) begin : north_south_switches
      n_switch n_switch
      (
        .p       (p[(16*23)+(j*6)+5:(16*23)+(j*6)]),
        .g_clr   (g_clr),
        .g_clk   (g_clk),
        .clk_in  (clk_in[j]),
        .nc_out  (nc_out[0][j]),
        .n4_in   (n4_in[j]),
        .ps4     (ps4_in[j]),
        .clk_out (clk_out[j]),
        .n_out   (n_out[j]),
        .n4_out  (n4_out[j]),
        .pn4     (pn4_out[j])
      );

      s_switch s_switch
      (
        .p       (p[(16*23)+(4*6)+(j*6)+5:(16*23)+(4*6)+(j*6)]),
        .g_clr   (g_clr),
        .g_clk   (g_clk),
        .clr_in  (clr_in[j]),
        .sc_out  (sc_out[3][j]),
        .s4_in   (s4_in[j]),
        .pn4     (pn4_in[j]),
        .clr_out (clr_out[j]),
        .s_out   (s_out[j]),
        .s4_out  (s4_out[j]),
        .ps4     (ps4_out[j])
      );
    end

    for (i=0; i<=3; i=i+1) begin : east_west_switches
      e_switch e_switch
      (
        .p       (p[(16*23)+(2*4*6)+(i*4)+3:(16*23)+(2*4*6)+(i*4)]),
        .ec_out  (ec_out[i][3]),
        .e4_in   (e4_in[i]),
        .pw4     (pw4_in[i]),
        .e_out   (e_out[i]),
        .e4_out  (e4_out[i]),
        .pe4     (pe4_out[i])
      );

      w_switch w_switch
      (
        .p       (p[(16*23)+(2*4*6)+(4*4)+(i*4)+3:(16*23)+(2*4*6)+(4*4)+(i*4)]),
        .wc_out  (wc_out[i][0]),
        .w4_in   (w4_in[i]),
        .pe4     (pe4_in[i]),
        .w_out   (w_out[i]),
        .w4_out  (w4_out[i]),
        .pw4     (pw4_out[i])
      );
    end
  endgenerate

endmodule

`endif /* BLOCK_4X4_SV */
